import { useScriptTag, useResizeObserver } from "@vueuse/core";
import { defu } from "defu";
import {
  inject,
  provide,
  ref,
  defineComponent,
  h,
  onBeforeMount,
  onMounted,
  onBeforeUnmount,
  getCurrentInstance,
  shallowRef
} from "vue-demi";
import { useState, useNuxtApp } from "#app";
const getKey = () => Number(Math.random().toString().slice(3)).toString(36);
function createInjectionState(composable) {
  const key = Symbol("UnityInjectionState");
  const useProvidingState = (...args) => {
    provide(key, composable(...args));
  };
  const useInjectedState = () => injectWithSelf(key);
  return [useProvidingState, useInjectedState];
}
const [useProvideNuxtUnity, useNuxtUnity] = createInjectionState(
  (initialValue, callback) => {
    const { $nuxtUnityEvent } = useNuxtApp();
    const unity = shallowRef(initialValue);
    const loaded = ref(false);
    const loading = ref(false);
    const quitted = ref(false);
    const error = ref();
    const providerId = useState("unityProviderId", getKey);
    function SendMessage(gameObject, method, param) {
      if (!loaded.value || !unity.value) {
        return;
      }
      if (param) {
        unity.value.SendMessage(gameObject, method, typeof param === "object" ? JSON.stringify(param) : param);
      } else {
        unity.value.SendMessage(gameObject, method);
      }
    }
    function done(unityInstance) {
      if (loaded.value) {
        return;
      }
      unity.value = unityInstance;
      loaded.value = true;
      loading.value = false;
      if (callback) {
        callback(unityInstance);
      }
    }
    async function quit() {
      if (!loaded.value || !unity.value) {
        return;
      }
      await unity.value.Quit().then(() => {
        quitted.value = true;
        $nuxtUnityEvent.emit("nuxt-unity:quit", providerId.value, unity.value);
      });
    }
    const UnityComponent = defineComponent({
      name: "NuxtUnityContainer",
      props: {
        width: {
          type: String,
          required: true
        },
        height: {
          type: String,
          required: true
        },
        unityLoader: {
          type: String,
          required: true
        },
        onProgress: {
          type: Function,
          default: (progress) => {
            console.log(`unity loading... ${progress * 100}%`);
          }
        },
        config: {
          type: Object,
          required: true
        },
        resizable: {
          type: Boolean,
          default: true
        },
        refreshDelayTime: {
          type: Number,
          default: 200
        },
        resetOnRefresh: {
          type: Boolean,
          default: false
        },
        onQuit: {
          type: Function,
          default: () => {
            console.log("quit unity");
          }
        }
      },
      emits: ["loading", "loaded", "error", "quitting", "quit"],
      setup(props, context) {
        const unityLoaderLoaded = ref(false);
        const canvasWidth = ref(props.width);
        const canvasHeight = ref(props.height);
        const container = shallowRef();
        const canvas = shallowRef();
        const config = defu(props.config, {
          dataUrl: "Build.data",
          frameworkUrl: "Build.framework.js",
          codeUrl: "Build.wasm",
          companyName: "DefaultCompany",
          productName: "UnityApp",
          productVersion: "0.1",
          showBanner: false,
          onQuit: props.onQuit
        });
        function instantiate() {
          if (typeof window.createUnityInstance === "undefined" && unityLoaderLoaded.value) {
            const _error = "Failed to load unityloader";
            error.value = new Error(_error);
            context.emit("error", _error);
            return;
          }
          if (loading.value) {
            return;
          }
          loading.value = true;
          context.emit("loading");
          const canvas2 = document.querySelector(`#unity-canvas-${providerId.value}`);
          window.createUnityInstance(canvas2, config, props.onProgress).then((unityInstance) => {
            done(unityInstance);
            $nuxtUnityEvent.emit("nuxt-unity:ready", providerId.value, unityInstance);
            context.emit("loaded");
          }).catch((message) => {
            error.value = new Error(message);
            context.emit("error");
          });
        }
        const { load } = useScriptTag(
          props.unityLoader,
          () => {
            unityLoaderLoaded.value = true;
            instantiate();
          },
          { manual: true }
        );
        onBeforeMount(() => {
          load();
        });
        onMounted(() => {
          let debounceTimer;
          useResizeObserver(container, (entries) => {
            if (!props.resizable) {
              return;
            }
            const entry = entries[0];
            const { clientWidth, clientHeight } = entry.target;
            if (debounceTimer) {
              clearTimeout(debounceTimer);
              debounceTimer = null;
            }
            if (props.resetOnRefresh) {
              canvasWidth.value = "0px";
              canvasHeight.value = "0px";
            }
            debounceTimer = setTimeout(() => {
              canvasWidth.value = clientWidth + "px";
              canvasHeight.value = clientHeight + "px";
            }, props.refreshDelayTime);
          });
        });
        onBeforeUnmount(() => {
          if (!unity.value) {
            return;
          }
          context.emit("quitting");
          if (unity.value) {
            quit().then(() => props.onQuit()).then(() => {
              context.emit("quit");
              unity.value = null;
              container.value?.remove();
            });
          }
        });
        return { canvasWidth, canvasHeight, container, canvas };
      },
      render() {
        return h(
          "div",
          {
            id: `unity-container-${providerId.value}`,
            "provider-id": providerId.value,
            class: "unity-container",
            ref: "container"
          },
          [
            h(
              "canvas",
              {
                id: `unity-canvas-${providerId.value}`,
                class: "unity-canvas",
                style: {
                  width: this.canvasWidth,
                  height: this.canvasHeight,
                  display: "block"
                },
                ref: "canvas"
              },
              []
            ),
            h("div", { class: "unity-extra-content" }, this.$slots.default?.())
          ]
        );
      }
    });
    return {
      unity,
      loading,
      loaded,
      quitted,
      quit,
      error,
      SendMessage,
      providerId,
      NuxtUnity: UnityComponent
    };
  }
);
export { useProvideNuxtUnity, useNuxtUnity };
export function useNuxtUnityOrThrow() {
  const store = useNuxtUnity();
  if (store == null) {
    throw new Error("Please call `useProvideUnity` on the appropriate parent component");
  }
  return store;
}
function injectWithSelf(symbol, def = void 0) {
  const vm = getCurrentInstance();
  return vm?.provides[symbol] || inject(symbol, def);
}
